function Eval(node)

%  This function evaluates the discrepancies for the tree branching from
%  'node'. They are accumulated in the global vector 'discs'
%  The discrepancies are forward-looking, from Now to Next, so the last period in which they
%  are defined is T-1

global T Ns
global Successors  Predecessor Values Periods States typeTrans Trans Ergodic
global discLoc discs ifStore dValues dNow dNext

period=Periods(node);
first=period==1;
N=Ns(period+1);

%  Pick the transition probability from 'node' to 'Successors(node,:)'.
%  This is a row vector.

switch typeTrans(period)
    case 1  %  from N=1 to a higher value
        tran=Ergodic';  %  this is normally only from period 1 to period 2
    case 2  %  from N to N, N>1
        tran=Trans(States(node),:);  %  the row corresponding to the state of 'node'
    case 3  %  from N>1 to 1
        tran=1;  %  the transition to a single possible node is certain
    case 4  %  from 1 to 1
        tran=1;
end

%  Compute the discrepancies corresponding to 'node' and 'Successors(node,:)'.

if period>1
    thenVars=Values(Predecessor(node),:);
else
    thenVars=[];
end
nowVars=Values(node,:);
DestNodes=Successors(node,1:N);
laterVars=Values(DestNodes,:);

discNode=Model(thenVars,nowVars,laterVars,tran,N,first,period,States(node));
ndisc=numel(discNode);  %  Note that Model determines the number of discrepancies
discs(discLoc:discLoc+ndisc-1)=discNode;
discLoc=discLoc+ndisc;

%  Store calculated values of derived variables

if ifStore
    numNow=numel(dNow);
    numNext=size(dNext,2);
    dValues(node,1:numNow)=dNow;
    if numel(DestNodes)==size(dNext,1)
        dValues(DestNodes',numNow+1:numNow+numNext)=dNext;  %  Note that the Next values are stored with the Next nodes
    else
        disp 'Discrepancy in Eval'
        DestNodes
        dNext
        N
        period
        Periods(DestNodes)
        stop
    end
end

%   If there is a non-terminal node among the successors, evaluate it

if period<T-1 %  if 'period' is T-1, there are no further nodes to evaluate
    for n=1:N
        Eval(Successors(node,n)); %  move to next period along this branch
    end
end

