function Grow(node)

%  This function calculates growth rates for the tree branching from
%  'node'. They are accumulated in the global matrix 'Growth' which has the
%  same layout as Values
%  Growth(node,var) contains the growth rate from the previous period to the period of the node
%  Growth(1,:) is undefined. The next Ns(1) rows of Growth contain the growth rates from period
%  1 to period 2, and so on.

global T Ns 
global Successors Values Periods Growth

period=Periods(node);
N=Ns(period+1);

nowVars=Values(node,:); %  Earlier period values at this node, a row vector by variable
DestNodes=Successors(node,1:N);
laterVars=Values(DestNodes,:);  %  Later period values, a matrix 
for succ=1:N
    gr=real(log(laterVars(succ,:)./nowVars));
    newRow=Successors(node,succ);
    Growth(newRow,:)=gr;
end

%   If there is a non-terminal node among the successors, evaluate it

if period<T-1 %  if 'period' is T-1, there are no further nodes to evaluate
    for n=1:N
        nextNode=Successors(node,n);
        Grow(nextNode); %  move to next period along this branch
    end
end

