function Jacob(node)

%  This function locates the non-zero elements of the Jacobian for the tree branching from
%  'node'. They are accumulated in the global vector 'Jp'

global T  Ns
global Successors  Predecessor Values Periods Updateds States  typeTrans Trans Ergodic
global discLoc locEq EqNodeMap

if Updateds(node)  %  Don't do anything if this node is already completed
    return
end

period=Periods(node);
first=period==1;
N=Ns(period+1);

%  Pick the transition probability from 'node' to 'Successors(node,:)'.
%  This is a row vector.

switch typeTrans(period)
    case 1  %  from N=1 to a higher value
        tran=Ergodic';  %  this is normally only from period 1 to period 2
    case 2  %  from N to N, N>1
        tran=Trans(States(node),:);  %  the row corresponding to the state of 'node'
    case 3  %  from N>1 to 1
        tran=1;  %  the transition to a single possible node is certain
    case 4  %  from 1 to 1
        tran=1;
end

%  Compute the discrepancies corresponding to 'node' and 'Successors(node,:)'.

if period>1
    thenVars=Values(Predecessor(node),:);
else
    thenVars=Values(1,:);  %  not actually used
end
nowVars=Values(node,:);
laterVars=Values(Successors(node,1:N),:);
discNode=Model(thenVars,nowVars,laterVars,tran,N,first,period,States(node));
ndisc=numel(discNode);
for add=1:ndisc
    locEq=locEq+1;
    EqNodeMap(locEq,:)=[node,discLoc+add-1];
end
discLoc=discLoc+ndisc;

%   If there is a non-terminal node among the successors, evaluate it

if period<T-1 %  if 'period' is T-1, there are no further nodes to evaluate
    for n=1:N
        Jacob(Successors(node,n)); %  move to next period along this branch
    end
    %  At this point, all the successors have been evaluated and we move on
    %  to the next branch for the next higher 'n'.
end
Updateds(node)=1;
