function discs=Model(thenVars,nowVars,laterVars,tran,N,first,period,state)
%  This function defines the model by calculating discrepancies in material
%  balance and the Euler equation
%  Variables stored by node: A, L, c, k
%  Calculate discrepancies in the Later values of the material balance
%  equation and the discrepancies in the Euler equation as of Now.

%  Functions calling Model: Eval, Jacob

global alpha delta beta sigma kappa omega uStar popGrowth dNow dNext ifStore
global dks dns zs gs dfs bs

nextStates=(1:N);  %  list of states in Next

%  Employment

LNow=nowVars(2);
nBar=(1-uStar)*LNow;  %  normal employment with u = uStar
kNow=nowVars(4);
AsNow=nowVars(1);
AsNext=laterVars(:,1);
dn=dns(period,state);
b=bs(period,state);
gsNext=gs(period,nextStates)';
z=zs(period,state);
df=dfs(period,state);
cNow=nowVars(3);
csNext=laterVars(:,3);
if first
    nNow=nBar;
    ms=ones(N,1);
else
    if b==0  %  no ZLB
        nNow=nBar*exp(-(omega/(1+alpha*omega))*(alpha*log(z)+dn));
    end
end
 
% Material balance

ksNext=laterVars(:,4);
if b==0  %  no ZLB
    ysNext=AsNext*kNow^alpha*nNow^(1-alpha);
    dLM=ksNext-(1-delta)*kNow-ysNext+csNext+gsNext;
    discs(1:N,1)=dLM;
else
      ysNext=ksNext-(1-delta)*kNow-+csNext+gsNext;  %  with ZLB, output is a residual
      %  Determine employment from output here when implementing ZLB
end

%  Euler equation

if first
    qNow=1;
    Rks(1:N,1)=1;
else
    kThen=thenVars(4);
    qNow=kappa*(kNow/kThen-1)+1;
    qsNext=kappa*(ksNext/kNow-1)+1;
    Rks=(alpha*AsNext*kNow^(alpha-1)./zs(period+1,nextStates)'+(1-delta)*qsNext)/qNow;
    ms=beta*(exp(-popGrowth)*csNext/cNow).^(-1/sigma);  %  per caput growth adjusted
    dEE=tran*(Rks.*ms)-dks(period,state)-1;
    discs(N+1)=dEE;
end

%  Real short rate

Em=tran*ms;
rf=(1+df)/Em-1;
if b==1  %  ZLB
    %  generate rf residuals here when implementing ZLB
end

%  Store derived variables

if ifStore
    rks=Rks-1;
    Erk=tran*rks;  
    u=1-nNow/LNow;
    dNow=[nNow,u,qNow,rf,Erk];
    invs=ksNext-kNow+delta*kNow;
    dNext=[ms,rks,invs,ysNext];
    if size(dNext,1)~=N
        N
        ms
        rks
        dNext
        save Model
    end
end
