%  Paths
%  Paths(nPath,period) is the state
%  Path calculation specifications
%  Path tracks an assigned sequence of states
%  PathC computes the moments of the paths starting from a second-period
%  state specified in column 2 of Paths.

%  Values for the sequence of states

clc
NPaths=size(Paths,1);
PathValues=[]; %   Moments along a path
PathValuesC=[];  %  Moments of levels and growth rates, conditional on a designated state in period 2

for nPath=1:NPaths
    PathValues=[PathValues;Paths(nPath,:)];  %  label the results
    PathValuesC=[PathValuesC;Paths(nPath,:)]; 
    Block=[];  %  Block accumulates the values
    node=1;  %  node corresponds to the period and state in Paths; always starts with 1
    for period=1:T
        Block=[Block,Values(node,:)'];  %  Put current values in Block
        if period<T  % set up the next node
            N=Ns(period+1);
            state=Paths(nPath,period+1);
            if state>N
                disp 'Bad state in path specification'
                return
            end
            node=Successors(node,state);  %  update node to next period
        end
    end
    PathValues=[PathValues;Block];  %  Stack this set of values into PathValues
    
    %  Calculations of moments conditional on a given state in period 2
    
    stateCond=Paths(nPath,2);  %  conditioning state in period 2
    Novar=size(Values,2);    
    BlockC=[Values(1,:)';zeros(Novar,1)];  %  Period 1 level moments, no standard deviations
    BlockCG=[zeros(2*Novar,1)];  %  Period 1 growth moments, undefined
    nodeC=Successors(1,stateCond); %  node number for period 2
    BlockC=[BlockC,[Values(nodeC,:)';zeros(Novar,1)]];  %  Period 2 moments
    BlockCG=[BlockCG,[Growth(nodeC,:)';zeros(Novar,1)]];  %  Period 2 growth moments, no standard deviations
    StartC=nodeC;
    EndC=nodeC;
    for t=3:T
        StartC=Successors(StartC,1); %  first node succeeding previous first node
        EndC=Successors(EndC,Ns(t)); %  last node succeeding previous last node
        prbs=Probs(StartC:EndC)'/sum(Probs(StartC:EndC));  %  distribution within the successor nodes
        for var=1:Novar
            meansC(var,1)=prbs*Values(StartC:EndC,var);
            means2C=prbs*Values(StartC:EndC,var).^2;
            sdsC(var,1)=real(sqrt(means2C-meansC(var)^2));
            meansCG(var,1)=prbs*Growth(StartC:EndC,var);
            means2CG=prbs*Growth(StartC:EndC,var).^2;
            sdsCG(var,1)=real(sqrt(means2CG-meansCG(var)^2));
        end
        BlockC=[BlockC,[meansC;sdsC]];
        BlockCG=[BlockCG,[meansCG;sdsCG]];
    end
    PathValuesC=[PathValuesC;BlockC;BlockCG];   %  stack these values into PathValuesC
end
disp 'IRF calcs completed'