clc
clear all

global N Nmax nextNode Created Ns nNodes ifStore dValues 
global Predecessor Successors Probs Periods Updateds States typeTrans Ergodic
global unks unksLow unksHi Values Starts Ends T locEq EqNodeMap Growth firstTran

SamePattern=true;  %  Jacobian pattern matrix 
NPaths=[];
params;  %  Set problem-specific parameters

%  Ergodic distribution

NMarkov=size(Trans,1);
Novar=numel(unksHi);
Tran1=Trans-eye(NMarkov);
TranA=[Tran1(:,1:NMarkov-1),ones(NMarkov,1)];
TranAinv=TranA^(-1);
Ergodic=TranAinv(NMarkov,:)';
N=NMarkov;  %  maximum value
Nmax=N;
Created=[];
firstTran=Ergodic;  

T=numel(Ns);
nNodes=0;
Add=1;
for t=1:T
    Add=Add*Ns(t);
    nNodes=nNodes+Add;
    numNodes(t)=nNodes;
end
nextNode=1;

NumberOfNodes=nNodes

Ttypes=[4,1;3,2];
for t=1:T-1
    typeTrans(t)=Ttypes(min(2,Ns(t)),min(2,Ns(t+1)));
end

%  Node data:

%Successors=zeros(nNodes,N);
% Values=zeros(nNodes,nVars);
Probs=zeros(nNodes,1);
Periods=zeros(nNodes,1);
States=zeros(nNodes,1);

%  Set up the nodes

disp 'Setting up nodes'
NewEntries=1;
node=1;  %  current node
Periods(1)=1;
Starts(1)=1;  %  node location of first value for a period
Ends(1)=1;  %  node location of last value for a period
Snode=2;  %  first successor node
for period=1:T-1
    Starts(period+1)=Snode;
    NewEntries=NewEntries*Ns(period);
    for entry=1:NewEntries
        eSize=Ns(period+1);
        Successors(node,1:eSize)=Snode:Snode+eSize-1;
        Predecessor(Snode:Snode+eSize-1,1)=node;
        Periods(Snode:Snode+eSize-1)=period+1;
        node=node+1;
        Snode=Snode+eSize;
    end
    Ends(period+1)=Snode-1;
end   

%  Period 1

Probs(1)=1;
States(1)=1;

%  Remaining nodes

Updateds=zeros(nNodes,1);
makeNode(1);

%  Fixed and initial values

disp 'Fixed and initial values'
Updateds=zeros(nNodes,1);
Initial(1);

%  Put starting values into the vector 'xs'

disp 'Starting values'
low=1;
VarNodeMap=[];
for var=unks
    rlow=Starts(unksLow(var)+1);% Starts is node location of first value for a period
    rhigh=Ends(T-unksHi(var));%  Ends is node location of last value for a period
    range=rlow:rhigh;
    nValues=numel(range);
    high=low+nValues-1;
    xs(low:high,1) = Values(range,var);
    %  VarNodeMap: (1) var is unknown variable number in Values, (2) rlow is the
    %  node of the starting period, (3) rhigh is the node of the ending
    %  period, (4) low is starting position in the vector of unknowns, x,
    %  (5) high is the ending position in x
    VarNodeMap=[VarNodeMap;[var,rlow,rhigh,low,high]];
    Offset(var)=low-rlow;  %  the offset to add to a node number to find the location of the
    %  value of the variable in the vector of unknowns
    low=low+nValues;
end
MaxFunEvals=30*high;
if ~SamePattern
%  Build the sparsity pattern for the Jacobian to save a lot of time for
%  larger problems. This version makes the assumption that every equation
%  involving a node and its successors is a function of all of the
%  successors. This is true for Euler equations but not for laws of motion.

disp 'Preliminary sparsity analysis'
Updateds=zeros(nNodes,1);
EqNodeMap=zeros(high,2);  %  Will contain a row for each discrepancy, with the originating node number and
%  the location in the full discrepancy vector of the discrepancy
locEq=0;
JacPat(xs);  %  form locEq and EqNodeMap from Model, via Jacob(node)
Rows=zeros(locEq*numel(unks)*(N+1),1);
Cols=Rows;
locJ=0;
for loc=1:locEq  %  create a Jacobian presence entry for each discrepancy x variable
    node = EqNodeMap(loc,1);  %  originating node
    row =  EqNodeMap(loc,2);  %  the row identifies the equation
    for var=unks  %  entries for originating node, Now variables
        col = node+Offset(var);  %  the column identifes the variable
        if col>0 && col <= high  %  avoid non-existent unknowns
            locJ=locJ+1;
            Rows(locJ)=row;
            Cols(locJ)=col;
        end
    end
    
    if node>1
        nodePred = Predecessor(node);  %  predecessor node
        for var=unks  %  entries for predecessor node, Then variables
            col = nodePred+Offset(var);
            if col>0 && col <= high  %  avoid non-existent unknowns
                locJ=locJ+1;
                Rows(locJ)=row;
                Cols(locJ)=col;
            end
        end
    end
    
    for succ=1:N  %  create entries for Later variables
        Snode=Successors(node,succ);
        if Snode==0
            break
        end
        for var=unks  %  entries for succeeding node
            col = Snode+Offset(var);  %  the column identifes the variable
            if col>0 && col <= high  %  avoid non-existent unknowns
                locJ=locJ+1;
                Rows(locJ)=row;
                Cols(locJ)=col;
            end
        end
    end
end
Rows=Rows(1:locJ);
Cols=Cols(1:locJ);
JP=sparse(Rows,Cols,ones(locJ,1),high,high);
SizeOfPrelimJacobian=locJ

%  Preliminary evaluation

disp 'First iteration to generate improved sparsity matrix'
ifStore=true;
Updateds=zeros(nNodes,1);
InitialDiscs=SPMsolve(xs);
% return

%  Solve

ifStore=false;
MaxIter=0;
options=optimset('JacobPattern',JP,'Display','iter','MaxFunEvals',MaxFunEvals,'MaxIter',MaxIter,'Algorithm','trust-region-reflective','TolFun',.00000000001,'TolX',.000000001);
%  Improve the Jacobian pattern with one iteration
[xN,errors,exitflag,output,JPbetter]=fsolve(@SPMsolve,xs,options);

else  %  SamePattern
    load SPM J xN
    JPbetter=J;
end

SizeOfBetterJacobian=size(JPbetter,1)

%  Iterate with improved pattern

disp 'Iterate with improved pattern'
MaxIter=10;
tic;
options=optimset('JacobPattern',JPbetter,'Display','iter','MaxFunEvals',MaxFunEvals,'MaxIter',MaxIter,'Algorithm','trust-region-reflective','TolFun',.00000000001,'TolX',.000000001);
[xN,errors,exitflag,output,J]=fsolve(@SPMsolve,xN,options);
condJ=condest(J);
Jacobian_condition=condJ
tolerance=std(errors)
MinutesToSolve=toc/60

%  Calculate other variables

disp 'Calculate variables and tabulate'
ifStore=true;
finalDiscs=SPMsolve(xN);
Values=[Values,dValues];
Novar=size(Values,2);

%  Means and standard deviations

means=zeros(T,Novar);
for t=1:T
    for var=1:Novar
        means(t,var)=Probs(Starts(t):Ends(t))'*Values(Starts(t):Ends(t),var);
        means2=Probs(Starts(t):Ends(t))'*Values(Starts(t):Ends(t),var).^2;
        sds(t,var)=real(sqrt(means2-means(t,var)^2));
    end
end

%  Growth rates

Growth=zeros(numNodes(T),Novar);
%   FinVals=zeros(numNodes(T-1),3+4*N);
Grow(1);  %  Put growth rates in Growth for 2:T 
Growth2=Growth.^2;  
meansG=zeros(T-1,1);
sdsG=zeros(T-1,1);
for t=2:T
    for var=1:Novar
        meansG(t,var)=Probs(Starts(t):Ends(t))'*Growth(Starts(t):Ends(t),var);
        meansG2=Probs(Starts(t):Ends(t))'*Growth(Starts(t):Ends(t),var).^2;
        sdsG(t,var)=real(sqrt(meansG2-meansG(t,var)^2));
    end
end

MGS=[means;sds;meansG;sdsG];

ValuesLabeled=[Periods,States,Values];
save SPM J xN

PathCalc

