function makeNode(node)

%  This function sets up the tree branching from 'node'.
%  It creates the N nodes that succeed 'node'. For each one, it calls itself to create
%  its successors.
%  'node' refers to an existing node for which As, Probs, and States have been entered. 
%  'period' is the period for the originating node, 'node'.
%  'firstTran' is set of transtion probabilities out of the single state of
%  period 1. For a standard solution, firstTran=Ergodic. For calculating an
%  IRF, it selects one or more states to simulate a particular shock.

global T nextNode Created Ns
global Successors Probs Periods Updateds States Trans typeTrans firstTran

% if Updateds(node)  %  Don't do anything if this node is already completed
%     return
% end

%  What happens in this loop:
%  This function is only called when a node is to be filled out.
%  This involves recording the state and the probability for
%  the new node. After that, if this is not creating a terminal node (that
%  is, 'period' is less than T-1), call this function to fill out a successor
%  node

period=Periods(node);
for n=1:Ns(period+1)
    nextNode = Successors(node,n);
    States(nextNode)=n;
    switch typeTrans(period)
        case 1  %  from N=1 to a higher value
            Probs(nextNode)=firstTran(n);
        case 2  %  from N to N, N>1
            Probs(nextNode)=Trans(States(node),n)*Probs(node);
        case 3  %  from N>1 to 1
            Probs(nextNode)=Probs(node);
        case 4  %  from 1 to 1
            Probs(nextNode)=Probs(node);
    end
    Created=[Created;[node,n,nextNode,period,Probs(nextNode)]];
    if period<T-1 %  if 'period' is T-1, the just-created node is terminal
        makeNode(nextNode);
    end
end
% Updateds(node)=1;
