%  params

%  Supplies model-specific parameters for the model with unemployment
%  Variables stored by node: A, L, c, k

%  Variables in Values at conclusion: A, L, c, k, n, q, rf, Erk, m, rk, inv, y

global alpha delta kappa beta sigma omega popGrowth T uStar
global Ns Trans unks unksLow unksHi Maxdiscs 
global as ls dks dns dfs zs gs theta rkStar bs

slump=true;  %  toggle for special slump parameters
%  Driving forces to boost for slump analysis; order = dk, dn, ell, a, z, g, df; no boost for a (boost(4))
%                                                       1   2    3  4  5  6  7                                                      
% boost=ones(1,7);  %    all DFs boosted
boost=zeros(1,7);  %  no DF boosted
iboost=7;
boost(iboost)=1;  %  selected DF

alpha	=	0.35	;
delta	=	0.1	;
beta	=	1	;
sigma	=	0.5	;
kappa	=	2	;
uStar	=	0.05959  ;
omega	=	4	;
rkStar	=	0.23  ;
popGrowth	=	0.013768398	;

Inputs=xlsread('Figures and tables','Directory','G15:L27');
Trans=Inputs(1:6,:);

Nbig=size(Trans,1);
Ns=[1,6*ones(1,4),ones(1,10)];
T=size(Ns,2);

%  Driving forces,  d_k d_n l a d_f

DFs=Inputs(7:12,:);

%  Ergodic distribution

NMarkov=size(Trans,1);
Tran1=Trans-eye(NMarkov);
TranA=[Tran1(:,1:NMarkov-1),ones(NMarkov,1)];
TranAinv=TranA^(-1);
Ergodic=TranAinv(NMarkov,:)';

%  Set some DFs to constants

DFconsts=Inputs(13,:);
for df=1:5
    if DFconsts(df)==0
        DFs(:,df)=Ergodic'*DFs(:,df);
    end
end
 
%  Set up driving forces

DFsByPeriod=zeros(5,T,Nbig);
for period=1:T
    for var=1:5
        if Ns(period)==1
            DFsByPeriod(var,period,1)=Ergodic'*DFs(:,var);
        else
            for state=1:Ns(period)
                DFsByPeriod(var,period,state)=DFs(state,var);
            end
        end
    end
end

for period=1:T
    for state=1:Nbig
        dks(period,state)=DFsByPeriod(1,period,state);
        dns(period,state)=DFsByPeriod(2,period,state);
        ls(period,state)=DFsByPeriod(3,period,state);
        as(period,state)=DFsByPeriod(4,period,state);
        dfs(period,state)=DFsByPeriod(5,period,state);
    end
end

zs=ones(T,Nbig);
gs=zeros(T,Nbig);
bs=zeros(T,Nbig);  %  for future implementation of ZLB

if slump
    ergdks=dks(2,:)*Ergodic;
    ergdns=dns(2,:)*Ergodic;
    ergls=ls(2,:)*Ergodic;
    ergdfs=dfs(2,:)*Ergodic;
    slumpYrs=(2:5)';
    slumpState=5;
    if boost(1)==1;dks(slumpYrs,slumpState)=0.167;else dks(slumpYrs,slumpState)=ergdks;end
    if boost(2)==1;dns(slumpYrs,slumpState)=0.01957;else dns(slumpYrs,slumpState)=ergdns;end
    if boost(3)==1;ls(slumpYrs,slumpState)=0.00105;else ls(slumpYrs,slumpState)=ergls;end
    if boost(5)==1;zs(slumpYrs,slumpState)=1.03;end
    if boost(6)==1;gs(slumpYrs,slumpState)=-0.05;end
    if boost(7)==1;dfs(slumpYrs,slumpState)=-0.06;else dfs(slumpYrs,slumpState)=ergdfs;end
    %  'as' kept the same
end

Maxdiscs=max(Ns)+1;  %  maximum number of discrepancies calculated per node

%  Map the unknowns
%  unks() is a list of the positions of the unknown variables in the
%  'Values' matrix
%  unksLow is the offset from period 1 where the variable becomes
%  unknown--e.g., unksLow=1 means the variable becomes unknown in period 2
%  unksHi is the offset from period T when the variable is last unknown--eg
%  unksHi=1 means the last unknown period for the variable is T-1
%  Note that unksLow and unksHi have entries for all variables but only
%  those for the unknowns matter

unks=[3,4]; %  consumption and capital are the unknowns
unksLow=[0,0,1,1];  %  consumption and capital first unknown in period 2
unksHi=[0,0,0,1];  %  consumption last unknown in period T; capital in period T-1

%  Growth: gamma=growth rate of A; lambda=growth rate of L; theta=growth
%  rate of c and k. These are log growth rate--e.g., A'/A=exp(gamma).

gamma=log(exp(as(2,:))*Ergodic);
lambda=log(exp(ls(2,:))*Ergodic);
theta=gamma/(1-alpha)+lambda;

